#pragma rtGlobals=1		// Use modern global access method.

//step1: be sure you are in the datafolder corresponding to the ARPES data; run initalize
//step2: open, as single image, the frame where the dirac cones resemble points
//step3: put cursors around one point, then run "find_Cone_Positon"
//step4: append cone position to image or to stack, to check out whether they are all in the right place, in case not correct.
//step5: run create_Ellipse_Profile
//step6: quick profile? 
//step7: extract APRES profile averages in a circle around each point, R=3 pixels
//setp8: measure Fermi or TOVB and deterimen the correction of unisochromatism  
//step9: correct data: correct_ARPES_WF
//step10: convert to kinetic energy

Menu "Ellipse"
	"initialize_Ellipse"
	"find_Cone_Positon"
	"appendToimage_Cone_Position"
	"fit_Ellipse_ExpData"
	"create_Ellipse_Profile"
	"quick_ARPES_from_profile"
	"extract_ARPES_from_profile"
	"unisochrom_ARPES_WF"
	"correct_ARPES_WF"
	"calc_ARPES_2nd_derivative"
	"ARPES_2_KE"
	"Extract_profile_from_ARPES_WF"
End


Function initialize_Ellipse()
   make/n=0/o expcx, expcy
   edit expcx, expcy
End

Function find_Cone_Positon()
	String dfStr=SIS_getDataFolderName()
	CurveFit/NTHR=0/Q Gauss2D  ::$(dfStr):$dfStr[pcsr(A),pcsr(B)][qcsr(A),qcsr(B)] /D 
	wave wc=W_coef
	wave wex=expcx
	wave wey=expcy
	InsertPoints 0,1, wex,wey
	wex[0]=wc[2]
	wey[0]=wc[4]
end

Function appendToimage_Cone_Position()
	appendToGraph/T expcy vs expcx
	ModifyGraph mode=3
end

Function FitEllipse(w,x,y) : FitFunc
Wave w
Variable x
Variable y
//CurveFitDialog/
//CurveFitDialog/ Coefficients 4
//CurveFitDialog/ w[0] = a
//CurveFitDialog/ w[1] = b
//CurveFitDialog/ w[2] = x0
//CurveFitDialog/ w[3] = y0
return ((x-w[2])/w[0])^2 + ((y-w[3])/w[1])^2 - 1
End

Function fit_Ellipse_ExpData()
	Duplicate/o expcx ellipseYFit, ellipseXFit
	Make/D/O W_ellipse={180,180,250,250} // a, b, x0, y0 guess
	FuncFit/ODR=3 FitEllipse, W_ellipse /X={expcx, expcy} /XD={ellipseXFit,ellipseYFit}
	AppendToGraph/T ellipseYFit vs ellipseXFit	
	ModifyGraph mode(ellipseYFit)=4,marker(ellipseYFit)=8
end

Function create_Ellipse_Profile()
	Make/N=360/O cx,cy
	wave w=W_ellipse
	wave wx=cx
	wave wy=cy
	wx[0]=w[2]
	wy[0]= YEllipse(w[2],1)
	variable i=1
	variable R,theta
	do
		R=sqrt((wx[i-1]-w[2])^2+(wy[i-1]-w[3])^2)
		wx[i]=wx[0]+R*sin(i*Pi/180)
		wy[i]= YEllipse(wx[i],i)	
		i+=1
	while (i<360) 
	duplicate/o cx crx
	duplicate/o cy cry
	crx=round(cx)
	cry=round(cy)
	//Dowindow cy
	//if (V_flag==0)
	AppendToGraph/T cy vs cx
	//endif
end

Function YEllipse(x,i)
	variable x,i
	wave w=W_ellipse
	return w[3]-w[1]*sign(cos(i*Pi/180))*sqrt(1-((x-w[2])/w[0])^2)
End

Function quick_ARPES_from_profile() 
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=dfStr
	String winNameStr="RP"+dfStr
	String profileStr=dfStr+"_RPS"
	make/n=(360,dimSize($stackStr,2))/o $profileStr
	wave wp=$profileStr
	wp[][]=1
	make/n=((dimSize($stackStr,0)),(dimSize($stackStr,1)))/o/b/u tmpROI
	make/n=(dimSize($stackStr,2))/o tmpAVG
	ImageToFrontB(winNameStr,profileStr) 
	variable i=0,j=0
	wave crx,cry
	do
		//method 1
		imageTransform/Beam={(crx[i]),(cry[i])} getBeam $stackStr
		doupdate
		ImageTransform/G=(i)/D=W_Beam putrow $profileStr
		i+=1
	while (i<360)
end

Function ImageToFrontB(graphNameStr,imageNameStr) 
	String graphNameStr
	String imageNameStr
	Dowindow $(graphNameStr)
	if (V_flag==0)
		display
		Dowindow/C $(GraphNameStr)
		appendImage/B $imageNameStr
	else
		DoWindow/F $graphNameStr
	endif
	ModifyGraph margin(left)=57,margin(bottom)=57,margin(top)=14,margin(right)=14,width=420
	ModifyGraph height=180
	SetAxis/A/R left
	Label bottom "\\F'Arial'\\Z18Azimuthal angle (degrees)"
	ModifyGraph fSize=12
	ShowInfo
End

Function extract_ARPES_from_profile() 
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=dfStr
	String winNameStr="RP"+dfStr
	String profileStr=dfStr+"_RPS"
	make/n=(360,dimSize($stackStr,2))/o $profileStr
	wave wp=$profileStr
	wp[][]=1
	make/n=((dimSize($stackStr,0)),(dimSize($stackStr,1)))/o/b/u tmpROI
	make/n=(dimSize($stackStr,2))/o tmpAVG
	ImageToFrontB(winNameStr,profileStr) 
	variable i=0,j=0
	wave crx,cry
	do
		 tmpROI=0 
		 tmpROI=(sqrt((p-crx[i])^2+(q-cry[i])^2)>5)
		 doUpdate
		 j=0
		 do
			ImageStats/P=(j)/R=tmpROI $stackStr // calc avg in the ROI in the datafolder
			tmpAVG[j]=V_avg
			j+=1
		while (j<dimSize($stackStr,2))
		ImageTransform/G=(i)/D=tmpAVG putrow $profileStr
		i+=1
	while (i<360)
end

Function	unisochrom_ARPES_WF()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=dfStr
	String rpWinNameStr="RP"+dfStr
	String profileStr=dfStr+"_RPS"
	variable np=dimSize($profileStr,0)
	variable i=0
	make/n=4/o w_coef
	make/n=(np)/o unisochr_results, fit_unisochr
	variable sm=1
	prompt sm, "smoothing"
	variable thres=0.5
	prompt thres, "threshold (%)"
	variable order=10
	prompt order, "polinomial fit order"
	doPrompt "correct ARPES" sm	,thres,order
	if (V_flag==0)
		do
			ImageTransform/G=(i) getRow $profileStr
			unisochr_results[i]=arpesThreshold(W_ExtractedRow,sm,thres) 
			i+=1
		while(i<np)
       	CurveFit/NTHR=0/TBOX=768 poly (order),  unisochr_results /D=fit_unisochr
       	SIS_waveToFront(rpWinNameStr,"fit_unisochr")
       endif
End	

Function arpesThreshold(w,sm,x)
      wave w
      variable sm,x
	duplicate/o w tmp
	Smooth (sm), w
	//
	wavestats/Q w
	W*=100
	w/=V_max
	//
	tmp= w > x
	Wavestats/Q tmp
	return V_maxloc
end
     
Function	correct_ARPES_WF()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=dfStr
	String winNameStr="WF"+dfStr
	String wfStr=dfStr+"_WF"
	String profileStr=dfStr+"_RPS"
	variable np=dimSize($profileStr,0)
	variable nq=dimSize($profileStr,1)
	duplicate/o $profileStr $wfStr 
       if (waveExists(fit_unisochr)==1)
       	wave fit_unisochr
       	waveStats/Q fit_unisochr
       	print V_avg
       	variable i=0
      		do
			ImageTransform/G=(i) getRow $profileStr
			InsertPoints 0,round(V_avg-fit_unisochr[i]), W_ExtractedRow
			redimension/n=(nq) W_ExtractedRow
			ImageTransform/G=(i)/D=W_ExtractedRow putrow $wfStr
			i+=1
		while(i<np)
	endif
	ImageToFrontB(winNameStr,wfStr) 
	SetScale/P y 0,1,"", $wfStr
	SetScale/P x 0,1,"", $wfStr
	ModifyImage  $wfStr ctab= {0,*,Grays,0};
	ModifyImage  $wfStr log=0
	Label left ""
End		

Function calc_ARPES_2nd_derivative()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=dfStr
	String winNameStr="WF"+dfStr
	String wfStr=dfStr+"_WF"
	variable nq=dimSize($wfStr,1)
	variable i=0
	variable sm=1
	prompt sm, "smoothing"
	doPrompt "calculate 2nd derivative" sm
	if(V_Flag==0)
		do
			ImageTransform/G=(i) getCol $wfStr
			Smooth (sm), W_ExtractedCol
			Differentiate W_ExtractedCol
			Differentiate W_ExtractedCol
			wave w=W_ExtractedCol
			w*=-1
			ImageTransform/G=(i)/D=W_ExtractedCol putcol $wfStr
			i+=1
		while(i<nq) 
		ModifyImage $wfStr log=0
		ModifyImage $wfStr ctab= {*,*,Grays,0}
	endif
End

Function ARPES_2_KE()
	variable scanFrom=38.3
	prompt scanFrom, "scan start energy (eV)"
	variable scanStep=-0.025
	prompt scanStep, "scan energy step (eV)"
	variable memLEEMtransition=0.756551
	prompt memLEEMtransition, "energy of the MEM to LEEM transition"
	doPrompt "ARPES to kinetic energy" scanFrom,scanStep,memLEEMtransition
	if (V_flag==0)
		String dfStr=SIS_getDataFolderName()
		SetDataFolder root:$(dfStr)
		String wfStr=dfStr+"_WF"
		String graphStr="WF"+dfStr
		ImageToFrontB(graphStr,wfStr)
		SetScale/P y (scanFrom-memLEEMtransition),(scanStep),"", $wfStr
		ModifyGraph width=420,height=180
		ModifyGraph margin(left)=57
		Label bottom "\\F'Arial'\\Z18Azimuthal angle (degrees)"
		Label left "\\F'Arial'\\Z18Kinetic energy (eV)"
		ModifyGraph tkLblRot=0
		ModifyGraph tlOffset(left)=5
		ModifyGraph fSize=12
		SetAxis/A
		ModifyGraph nticks(left)=5,nticks(bottom)=15
		ModifyImage $wfStr ctab= {1,*,grays,0}
		ModifyImage $wfStr log=1
		ModifyGraph minor=1
		// create WF pq waves
		String pwfStr="WF"+dfStr+"_P"
		make/n=(dimSize($wfStr,0)+1)/o $pwfStr=p
		String qwfStr="WF"+dfStr+"_Q"
		make/n=(dimSize($wfStr,1)+1)/o $qwfStr=(scanFrom-memLEEMtransition)+p*scanStep
	endif
end		

Function Extract_profile_from_ARPES_WF()
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=dfStr
	String graphNameStr="PR"+dfStr
	String xwaveNameStr="PR"+dfStr+"_P"
	String wfStr=dfStr+"_WF"
	variable np=dimSize($wfStr,0)
	variable nq=dimSize($wfStr,1)
	if (stringMatch(num2Str(pcsr(A)),"NaN")!=1 && stringMatch(num2Str(pcsr(B)),"NaN")!=1)
		variable p1=pcsr(A)
		variable p2=pcsr(B)
		variable q1=qcsr(A)
		variable q2=qcsr(B)
		String waveNameStr="PR"+dfStr+"_"+num2Str(q1)
		ImageTransform/G=(q1) getCol $wfStr
		duplicate/o W_ExtractedCol $waveNameStr
		duplicate/o W_ExtractedCol $xwaveNameStr
		wave wx=$xwaveNameStr
		wx=p
		deletepoints 0,(p1),  $xwaveNameStr,$waveNameStr
		deletepoints (p2-p1+1),(np-p2),  $xwaveNameStr,$waveNameStr
		SIS_graphToFront(graphNameStr,waveNameStr,xwaveNameStr)
	endif
End